@@ -16,19 +16,23 @@
 #include <KLocalizedString>
 #include <KStatusNotifierItem>
 
-#include "client.h"
-#include "context.h"
-#include "pulseaudio.h"
-#include "source.h"
+#include <PulseAudioQt/Client>
+#include <PulseAudioQt/Context>
+#include <PulseAudioQt/Models>
+#include <PulseAudioQt/Server>
+#include <PulseAudioQt/Source>
 
-#include "volumeosd.h"
+using namespace PulseAudioQt;
+using namespace Qt::Literals::StringLiterals;
 
-using namespace QPulseAudio;
+constexpr QLatin1String OSD_DBUS_SERVICE = "org.kde.plasmashell"_L1;
+constexpr QLatin1String OSD_DBUS_PATH = "/org/kde/osdService"_L1;
 
 MicrophoneIndicator::MicrophoneIndicator(QObject *parent)
     : QObject(parent)
     , m_sourceModel(new SourceModel(this))
     , m_sourceOutputModel(new SourceOutputModel(this))
+    , m_osdDBusInterface(new OsdServiceInterface(OSD_DBUS_SERVICE, OSD_DBUS_PATH, QDBusConnection::sessionBus(), this))
     , m_updateTimer(new QTimer(this))
 {
     connect(m_sourceModel, &QAbstractItemModel::rowsInserted, this, &MicrophoneIndicator::scheduleUpdate);
@@ -113,7 +117,7 @@ void MicrophoneIndicator::update()
     if (allMuted) {
         iconName = QStringLiteral("microphone-sensitivity-muted");
     } else {
-        if (Source *defaultSource = m_sourceModel->defaultSource()) {
+        if (Source *defaultSource = m_preferredDevice.source()) {
             const int percent = volumePercent(defaultSource);
             iconName = QStringLiteral("microphone-sensitivity");
             // it deliberately never shows the "muted" icon unless *all* microphones are muted
@@ -205,40 +209,36 @@ void MicrophoneIndicator::toggleMuted()
 
 void MicrophoneIndicator::adjustVolume(int direction)
 {
-    Source *source = m_sourceModel->defaultSource();
+    Source *source = m_preferredDevice.source();
     if (!source) {
         return;
     }
 
-    const int step = qRound(5 * Context::NormalVolume / 100.0);
+    const int step = qRound(5 * normalVolume() / 100.0);
 
-    const auto newVolume = qBound(Context::MinimalVolume, //
+    const auto newVolume = qBound(minimumVolume(), //
                                   source->volume() + direction * step, //
-                                  Context::NormalVolume);
+                                  normalVolume());
 
     source->setVolume(newVolume);
-    source->setMuted(newVolume == Context::MinimalVolume);
+    source->setMuted(newVolume == minimumVolume());
 
     m_showOsdOnUpdate = true;
 }
 
 int MicrophoneIndicator::volumePercent(Source *source)
 {
-    return source->isMuted() ? 0 : qRound(source->volume() / static_cast<qreal>(Context::NormalVolume) * 100);
+    return source->isMuted() ? 0 : qRound(source->volume() / static_cast<qreal>(normalVolume()) * 100);
 }
 
 void MicrophoneIndicator::showOsd()
 {
-    if (!m_osd) {
-        m_osd = new VolumeOSD(this);
-    }
-
-    auto *preferredSource = m_sourceModel->defaultSource();
+    auto *preferredSource = m_preferredDevice.source();
     if (!preferredSource) {
         return;
     }
 
-    m_osd->showMicrophone(volumePercent(preferredSource));
+    m_osdDBusInterface->microphoneVolumeChanged(volumePercent(preferredSource));
 }
 
 QList<QModelIndex> MicrophoneIndicator::recordingApplications() const
